@@ -112,7 +112,8 @@ test_slot_info (Test *test, gconstpointer unused)
 			g_assert_cmpint (175, ==, token->hardware_version_minor);
 			g_assert_cmpint (85, ==, token->firmware_version_major);
 			g_assert_cmpint (185, ==, token->firmware_version_minor);
-			g_assert_cmpint (927623999, ==, token->utc_time);
+			g_assert_nonnull (token->utc_time);
+			g_assert_cmpint (927623999, ==, g_date_time_to_unix (token->utc_time));
 
 			gck_token_info_free (token);
 		}
@@ -156,7 +157,7 @@ test_slot_equals_hash (Test *test, gconstpointer unused)
 	g_object_unref (other_slot);
 
 	obj = g_object_new (G_TYPE_OBJECT, NULL);
-	g_assert_false (gck_slot_equal (test->slot, obj));
+	g_assert_false (gck_slot_equal (test->slot, (GckSlot *) obj));
 	g_object_unref (obj);
 
 	other_slot = g_object_new (GCK_TYPE_SLOT, "module", test->module, "handle", 8909, NULL);
@@ -182,7 +183,7 @@ test_slot_mechanisms (Test *test, gconstpointer unused)
 		gck_mechanism_info_free (info);
 	}
 
-	gck_mechanisms_free (mechs);
+	g_array_unref (mechs);
 }
 
 static void
