@@ -22,7 +22,6 @@
 #include "gcr-certificate.h"
 #include "gcr-certificate-extensions.h"
 #include "gcr-comparable.h"
-#include "gcr-icons.h"
 #include "gcr-internal.h"
 #include "gcr-subject-public-key.h"
 
@@ -69,15 +68,6 @@
  * The interface that implementors of #GcrCertificate must implement.
  */
 
-/**
- * GCR_CERTIFICATE_COLUMNS:
- *
- * The columns that are valid for a certificate. This is to be used with
- * the [class@TreeSelector] or [class@CollectionModel].
- *
- * This is an array of #GcrColumn, owned by the gcr library.
- */
-
 /*
  * The DER data in this structure is owned by the derived class.
  * It is only valid for the duration of the current call stack
@@ -104,12 +94,10 @@ static GBytes * _gcr_certificate_get_issuer_const (GcrCertificate *self);
 enum {
 	PROP_FIRST = 0x0007000,
 	PROP_LABEL,
-	PROP_MARKUP,
 	PROP_DESCRIPTION,
-	PROP_ICON,
-	PROP_SUBJECT,
-	PROP_ISSUER,
-	PROP_EXPIRY
+	PROP_SUBJECT_NAME,
+	PROP_ISSUER_NAME,
+	PROP_EXPIRY_DATE
 };
 
 /* -----------------------------------------------------------------------------
@@ -192,55 +180,6 @@ digest_certificate (GcrCertificate *self, GChecksumType type)
 	return digest;
 }
 
-/**
- * gcr_certificate_get_markup_text:
- * @self: a certificate
- *
- * Calculate a GMarkup string for displaying this certificate.
- *
- * Returns: (transfer full): the markup string
- */
-gchar *
-gcr_certificate_get_markup_text (GcrCertificate *self)
-{
-	gchar *label = NULL;
-	gchar *issuer;
-	gchar *markup;
-
-	g_object_get (self, "label", &label, NULL);
-	issuer = gcr_certificate_get_issuer_name (self);
-
-	if (issuer)
-		markup = g_markup_printf_escaped ("%s\n<small>Issued by: %s</small>", label, issuer);
-	else
-		markup = g_markup_printf_escaped ("%s\n<small>Issued by: <i>No name</i></small>", label);
-
-	g_free (label);
-	g_free (issuer);
-	return markup;
-}
-
-static void
-on_transform_date_to_string (const GValue *src, GValue *dest)
-{
-	static const gsize len = 256;
-	GDate *date;
-	gchar *result;
-
-	g_return_if_fail (G_VALUE_TYPE (src) == G_TYPE_DATE);
-
-	date = g_value_get_boxed (src);
-	g_return_if_fail (date);
-
-	result = g_malloc0 (len);
-	if (!g_date_strftime (result, len, "%x", date)) {
-		g_free (result);
-		result = NULL;
-	}
-
-	g_value_take_string (dest, result);
-}
-
 /* ---------------------------------------------------------------------------------
  * INTERFACE
  */
@@ -271,50 +210,32 @@ gcr_certificate_default_init (GcrCertificateIface *iface)
 		         g_param_spec_string ("description", "Description", "Description of object being rendered",
 		                              "", G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-		/**
-		 * GcrCertificate:markup:
-		 *
-		 * GLib markup to describe the certificate
-		 */
-		g_object_interface_install_property (iface,
-		         g_param_spec_string ("markup", "Markup", "Markup which describes object being rendered",
-		                              "", G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
-		/**
-		 * GcrCertificate:icon:
-		 *
-		 * An icon representing the certificate
-		 */
-		g_object_interface_install_property (iface,
-		         g_param_spec_object ("icon", "Icon", "Icon for the object being rendered",
-		                              G_TYPE_ICON, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
 		/**
 		 * GcrCertificate:subject:
 		 *
 		 * Common name part of the certificate subject
 		 */
 		g_object_interface_install_property (iface,
-		           g_param_spec_string ("subject", "Subject", "Common name of subject",
+		           g_param_spec_string ("subject-name", "Subject name", "Common name of subject",
 		                                "", G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 		/**
-		 * GcrCertificate:issuer:
+		 * GcrCertificate:issuer-name:
 		 *
 		 * Common name part of the certificate issuer
 		 */
 		g_object_interface_install_property (iface,
-		           g_param_spec_string ("issuer", "Issuer", "Common name of issuer",
+		           g_param_spec_string ("issuer-name", "Issuer name", "Common name of issuer",
 		                                "", G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 		/**
-		 * GcrCertificate:expiry:
+		 * GcrCertificate:expiry-date:
 		 *
 		 * The expiry date of the certificate
 		 */
 		g_object_interface_install_property (iface,
-		           g_param_spec_boxed ("expiry", "Expiry", "Certificate expiry",
-		                               G_TYPE_DATE, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+		           g_param_spec_boxed ("expiry-date", "Expiry date", "Certificate expiry date",
+		                               G_TYPE_DATE_TIME, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 		g_once_init_leave (&initialized, 1);
 	}
@@ -328,32 +249,6 @@ G_DEFINE_INTERFACE (GcrCertificate, gcr_certificate, GCR_TYPE_COMPARABLE);
  * PUBLIC
  */
 
-/**
- * gcr_certificate_get_columns: (skip)
- *
- * Get the columns appropriate for a certificate
- *
- * Returns: (transfer none): the columns
- */
-const GcrColumn*
-gcr_certificate_get_columns (void)
-{
-	static GcrColumn columns[] = {
-		{ "icon", /* later */ 0, /* later */ 0, NULL, 0 },
-		{ "label", G_TYPE_STRING, G_TYPE_STRING, NC_("column", "Name"),
-		  GCR_COLUMN_SORTABLE },
-		{ "issuer", G_TYPE_STRING, G_TYPE_STRING, NC_("column", "Issued By"),
-		  GCR_COLUMN_SORTABLE },
-		{ "expiry", /* later */ 0, G_TYPE_STRING, NC_("column", "Expires"),
-		  GCR_COLUMN_SORTABLE, on_transform_date_to_string },
-		{ NULL }
-	};
-
-	columns[0].property_type = columns[0].column_type = G_TYPE_ICON;
-	columns[3].property_type = G_TYPE_DATE;
-	return columns;
-}
-
 /**
  * gcr_certificate_compare:
  * @first: (nullable): the certificate to compare
@@ -421,7 +316,7 @@ gcr_certificate_get_der_data (GcrCertificate *self,
  * This will try to lookup the common name, orianizational unit,
  * organization in that order.
  *
- * Returns: the allocated issuer name, or %NULL if no issuer name
+ * Returns: (nullable): the allocated issuer name, or %NULL if no issuer name
  */
 gchar *
 gcr_certificate_get_issuer_name (GcrCertificate *self)
@@ -446,7 +341,7 @@ gcr_certificate_get_issuer_name (GcrCertificate *self)
  * The string returned should be freed by the caller when no longer
  * required.
  *
- * Returns: The allocated issuer CN, or %NULL if no issuer CN present.
+ * Returns: (nullable): The allocated issuer CN, or %NULL if no issuer CN present.
  */
 gchar*
 gcr_certificate_get_issuer_cn (GcrCertificate *self)
@@ -503,14 +398,14 @@ _gcr_certificate_get_issuer_const (GcrCertificate *self)
 /**
  * gcr_certificate_get_issuer_raw:
  * @self: a #GcrCertificate
- * @n_data: The length of the returned data.
+ * @n_data: (out): The length of the returned data.
  *
  * Get the raw DER data for the issuer DN of the certificate.
  *
  * The data should be freed by using g_free() when no longer required.
  *
- * Returns: (transfer full) (array length=n_data): allocated memory containing
- *          the raw issuer
+ * Returns: (transfer full) (array length=n_data) (nullable): allocated memory
+ *          containing the raw issuer
  */
 guchar *
 gcr_certificate_get_issuer_raw (GcrCertificate *self,
@@ -523,11 +418,13 @@ gcr_certificate_get_issuer_raw (GcrCertificate *self,
 	g_return_val_if_fail (n_data != NULL, NULL);
 
 	bytes = _gcr_certificate_get_issuer_const (self);
-	if (bytes == NULL)
+	if (bytes == NULL) {
+		*n_data = 0;
 		return NULL;
+	}
 
 	*n_data = g_bytes_get_size (bytes);
-	result = g_memdup (g_bytes_get_data (bytes, NULL), *n_data);
+	result = g_memdup2 (g_bytes_get_data (bytes, NULL), *n_data);
 	g_bytes_unref (bytes);
 
 	return result;
@@ -581,7 +478,7 @@ gcr_certificate_is_issuer (GcrCertificate *self, GcrCertificate *issuer)
  * The string returned should be freed by the caller when no longer
  * required.
  *
- * Returns: The allocated issuer DN of the certificate.
+ * Returns: (nullable): The allocated issuer DN of the certificate.
  */
 gchar*
 gcr_certificate_get_issuer_dn (GcrCertificate *self)
@@ -606,7 +503,7 @@ gcr_certificate_get_issuer_dn (GcrCertificate *self)
  * The string returned should be freed by the caller when no longer
  * required.
  *
- * Returns: The allocated subject CN, or %NULL if no subject CN present.
+ * Returns: (nullable): The allocated subject CN, or %NULL if no subject CN present.
  */
 gchar*
 gcr_certificate_get_subject_cn (GcrCertificate *self)
@@ -623,7 +520,7 @@ gcr_certificate_get_subject_cn (GcrCertificate *self)
  * This will try to lookup the common name, orianizational unit,
  * organization in that order.
  *
- * Returns: the allocated subject name, or %NULL if no subject name
+ * Returns: (nullable): the allocated subject name, or %NULL if no subject name
  */
 gchar *
 gcr_certificate_get_subject_name (GcrCertificate *self)
@@ -681,7 +578,7 @@ gcr_certificate_get_subject_part (GcrCertificate *self, const char *part)
  * The string returned should be freed by the caller when no longer
  * required.
  *
- * Returns: The allocated subject DN of the certificate.
+ * Returns: (nullable): The allocated subject DN of the certificate.
  */
 gchar*
 gcr_certificate_get_subject_dn (GcrCertificate *self)
@@ -714,14 +611,14 @@ _gcr_certificate_get_subject_const (GcrCertificate *self)
 /**
  * gcr_certificate_get_subject_raw:
  * @self: a #GcrCertificate
- * @n_data: The length of the returned data.
+ * @n_data: (out): The length of the returned data.
  *
  * Get the raw DER data for the subject DN of the certificate.
  *
  * The data should be freed by using g_free() when no longer required.
  *
- * Returns: (transfer full) (array length=n_data): allocated memory containing
- *          the raw subject
+ * Returns: (transfer full) (array length=n_data) (nullable): allocated memory
+ *          containing the raw subject
  */
 guchar *
 gcr_certificate_get_subject_raw (GcrCertificate *self, gsize *n_data)
@@ -733,11 +630,13 @@ gcr_certificate_get_subject_raw (GcrCertificate *self, gsize *n_data)
 	g_return_val_if_fail (n_data != NULL, NULL);
 
 	bytes = _gcr_certificate_get_subject_const (self);
-	if (bytes == NULL)
+	if (bytes == NULL) {
+		*n_data = 0;
 		return NULL;
+	}
 
 	*n_data = g_bytes_get_size (bytes);
-	result = g_memdup (g_bytes_get_data (bytes, NULL), *n_data);
+	result = g_memdup2 (g_bytes_get_data (bytes, NULL), *n_data);
 
 	g_bytes_unref (bytes);
 
@@ -750,16 +649,12 @@ gcr_certificate_get_subject_raw (GcrCertificate *self, gsize *n_data)
  *
  * Get the issued date of this certificate.
  *
- * The #GDate returned should be freed by the caller using
- * g_date_free() when no longer required.
- *
- * Returns: An allocated issued date of this certificate.
+ * Returns: (nullable): A issued date of this certificate.
  */
-GDate*
+GDateTime *
 gcr_certificate_get_issued_date (GcrCertificate *self)
 {
 	GcrCertificateInfo *info;
-	GDate *date;
 
 	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
 
@@ -767,13 +662,7 @@ gcr_certificate_get_issued_date (GcrCertificate *self)
 	if (info == NULL)
 		return NULL;
 
-	date = g_date_new ();
-	if (!egg_asn1x_get_time_as_date (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notBefore", NULL), date)) {
-		g_date_free (date);
-		return NULL;
-	}
-
-	return date;
+	return egg_asn1x_get_time_as_date_time (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notBefore", NULL));
 }
 
 /**
@@ -782,16 +671,12 @@ gcr_certificate_get_issued_date (GcrCertificate *self)
  *
  * Get the expiry date of this certificate.
  *
- * The #GDate returned should be freed by the caller using
- * g_date_free() when no longer required.
- *
- * Returns: An allocated expiry date of this certificate.
+ * Returns: (nullable): An expiry date of this certificate.
  */
-GDate*
+GDateTime *
 gcr_certificate_get_expiry_date (GcrCertificate *self)
 {
 	GcrCertificateInfo *info;
-	GDate *date;
 
 	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), NULL);
 
@@ -799,13 +684,7 @@ gcr_certificate_get_expiry_date (GcrCertificate *self)
 	if (info == NULL)
 		return NULL;
 
-	date = g_date_new ();
-	if (!egg_asn1x_get_time_as_date (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notAfter", NULL), date)) {
-		g_date_free (date);
-		return NULL;
-	}
-
-	return date;
+	return egg_asn1x_get_time_as_date_time (egg_asn1x_node (info->asn1, "tbsCertificate", "validity", "notAfter", NULL));
 }
 
 /**
@@ -842,14 +721,14 @@ gcr_certificate_get_key_size (GcrCertificate *self)
  * gcr_certificate_get_fingerprint:
  * @self: a #GcrCertificate
  * @type: the type of algorithm for the fingerprint.
- * @n_length: The length of the resulting fingerprint.
+ * @n_length: (out): The length of the resulting fingerprint.
  *
  * Calculate the fingerprint for this certificate.
  *
  * The caller should free the returned data using g_free() when
  * it is no longer required.
  *
- * Returns: (array length=n_length): the raw binary fingerprint
+ * Returns: (array length=n_length) (nullable): the raw binary fingerprint
  **/
 guchar *
 gcr_certificate_get_fingerprint (GcrCertificate *self, GChecksumType type, gsize *n_length)
@@ -862,8 +741,10 @@ gcr_certificate_get_fingerprint (GcrCertificate *self, GChecksumType type, gsize
 	g_return_val_if_fail (n_length != NULL, NULL);
 
 	sum = digest_certificate (self, type);
-	if (sum == NULL)
+	if (sum == NULL) {
+		*n_length = 0;
 		return NULL;
+	}
 
 	length = g_checksum_type_get_length (type);
 	g_return_val_if_fail (length > 0, NULL);
@@ -886,7 +767,7 @@ gcr_certificate_get_fingerprint (GcrCertificate *self, GChecksumType type, gsize
  * The caller should free the returned data using g_free() when
  * it is no longer required.
  *
- * Returns: an allocated hex string which contains the fingerprint.
+ * Returns: (nullable): an allocated hex string which contains the fingerprint.
  */
 gchar*
 gcr_certificate_get_fingerprint_hex (GcrCertificate *self, GChecksumType type)
@@ -917,14 +798,14 @@ gcr_certificate_get_fingerprint_hex (GcrCertificate *self, GChecksumType type)
 /**
  * gcr_certificate_get_serial_number:
  * @self: a #GcrCertificate
- * @n_length: the length of the returned data.
+ * @n_length: (out): the length of the returned data.
  *
  * Get the raw binary serial number of the certificate.
  *
  * The caller should free the returned data using g_free() when
  * it is no longer required.
  *
- * Returns: (array length=n_length): the raw binary serial number.
+ * Returns: (array length=n_length) (nullable): the raw binary serial number.
  */
 guchar *
 gcr_certificate_get_serial_number (GcrCertificate *self, gsize *n_length)
@@ -937,14 +818,16 @@ gcr_certificate_get_serial_number (GcrCertificate *self, gsize *n_length)
 	g_return_val_if_fail (n_length != NULL, NULL);
 
 	info = certificate_info_load (self);
-	if (info == NULL)
+	if (info == NULL) {
+		*n_length = 0;
 		return NULL;
+	}
 
 	bytes = egg_asn1x_get_integer_as_raw (egg_asn1x_node (info->asn1, "tbsCertificate", "serialNumber", NULL));
 	g_return_val_if_fail (bytes != NULL, NULL);
 
 	*n_length = g_bytes_get_size (bytes);
-	result = g_memdup (g_bytes_get_data (bytes, NULL), *n_length);
+	result = g_memdup2 (g_bytes_get_data (bytes, NULL), *n_length);
 
 	g_bytes_unref (bytes);
 	return result;
@@ -959,7 +842,7 @@ gcr_certificate_get_serial_number (GcrCertificate *self, gsize *n_length)
  * The caller should free the returned data using g_free() when
  * it is no longer required.
  *
- * Returns: an allocated string containing the serial number as hex.
+ * Returns: (nullable): an allocated string containing the serial number as hex.
  */
 gchar*
 gcr_certificate_get_serial_number_hex (GcrCertificate *self)
@@ -979,22 +862,6 @@ gcr_certificate_get_serial_number_hex (GcrCertificate *self)
 	return hex;
 }
 
-/**
- * gcr_certificate_get_icon: (skip)
- * @self: The certificate
- *
- * Get the icon for a certificate.
- *
- * Returns: (transfer full): the icon for this certificate, which should be
- *          released with g_object_unref()
- */
-GIcon *
-gcr_certificate_get_icon (GcrCertificate *self)
-{
-	g_return_val_if_fail (GCR_IS_CERTIFICATE (self), FALSE);
-	return g_themed_icon_new (GCR_ICON_CERTIFICATE);
-}
-
 /**
  * gcr_certificate_get_basic_constraints:
  * @self: the certificate
@@ -1066,10 +933,9 @@ gcr_certificate_mixin_emit_notify (GcrCertificate *self)
 
 	obj = G_OBJECT (self);
 	g_object_notify (obj, "label");
-	g_object_notify (obj, "markup");
-	g_object_notify (obj, "subject");
-	g_object_notify (obj, "issuer");
-	g_object_notify (obj, "expiry");
+	g_object_notify (obj, "subject-name");
+	g_object_notify (obj, "issuer-name");
+	g_object_notify (obj, "expiry-date");
 }
 
 /**
@@ -1081,7 +947,7 @@ gcr_certificate_mixin_emit_notify (GcrCertificate *self)
  * macro instead of this function.
  */
 void
-gcr_certificate_mixin_comparable_init (GcrComparableIface *iface)
+gcr_certificate_mixin_comparable_init (GcrComparableInterface *iface)
 {
 	iface->compare = gcr_certificate_compare;
 }
@@ -1101,18 +967,14 @@ gcr_certificate_mixin_class_init (GObjectClass *object_class)
 {
 	if (!g_object_class_find_property (object_class, "description"))
 		g_object_class_override_property (object_class, PROP_DESCRIPTION, "description");
-	if (!g_object_class_find_property (object_class, "markup"))
-		g_object_class_override_property (object_class, PROP_MARKUP, "markup");
 	if (!g_object_class_find_property (object_class, "label"))
 		g_object_class_override_property (object_class, PROP_LABEL, "label");
-	if (!g_object_class_find_property (object_class, "icon"))
-		g_object_class_override_property (object_class, PROP_ICON, "icon");
-	if (!g_object_class_find_property (object_class, "subject"))
-		g_object_class_override_property (object_class, PROP_SUBJECT, "subject");
-	if (!g_object_class_find_property (object_class, "issuer"))
-		g_object_class_override_property (object_class, PROP_ISSUER, "issuer");
-	if (!g_object_class_find_property (object_class, "expiry"))
-		g_object_class_override_property (object_class, PROP_EXPIRY, "expiry");
+	if (!g_object_class_find_property (object_class, "subject-name"))
+		g_object_class_override_property (object_class, PROP_SUBJECT_NAME, "subject-name");
+	if (!g_object_class_find_property (object_class, "issuer-name"))
+		g_object_class_override_property (object_class, PROP_ISSUER_NAME, "issuer-name");
+	if (!g_object_class_find_property (object_class, "expiry-date"))
+		g_object_class_override_property (object_class, PROP_EXPIRY_DATE, "expiry-date");
 
 	_gcr_initialize_library ();
 }
@@ -1169,22 +1031,16 @@ gcr_certificate_mixin_get_property (GObject *obj, guint prop_id,
 	case PROP_LABEL:
 		g_value_take_string (value, gcr_certificate_get_subject_name (cert));
 		break;
-	case PROP_SUBJECT:
+	case PROP_SUBJECT_NAME:
 		g_value_take_string (value, gcr_certificate_get_subject_name (cert));
 		break;
-	case PROP_ICON:
-		g_value_set_object (value, gcr_certificate_get_icon (cert));
-		break;
 	case PROP_DESCRIPTION:
 		g_value_set_string (value, _("Certificate"));
 		break;
-	case PROP_MARKUP:
-		g_value_take_string (value, gcr_certificate_get_markup_text (cert));
-		break;
-	case PROP_ISSUER:
+	case PROP_ISSUER_NAME:
 		g_value_take_string (value, gcr_certificate_get_issuer_name (cert));
 		break;
-	case PROP_EXPIRY:
+	case PROP_EXPIRY_DATE:
 		g_value_take_boxed (value, gcr_certificate_get_expiry_date (cert));
 		break;
 	default:
