@@ -21,7 +21,6 @@
 
 #include "config.h"
 
-#include "gcr-deprecated-base.h"
 #include "gcr-importer.h"
 #include "gcr-internal.h"
 #include "gcr-gnupg-importer.h"
@@ -49,23 +48,20 @@
  * additional item with a set of importers using the
  * [func@Importer.queue_and_filter_for_parsed].
  *
- * To start the import, use [method@Importer.import] or its async variants.
+ * To start the import, use [method@Importer.import_async].
  */
 
 /**
- * GcrImporterIface:
+ * GcrImporterInterface:
  * @parent: parent interface
  * @create_for_parsed: implementation of gcr_importer_create_for_parsed(), required
  * @queue_for_parsed: implementation of gcr_importer_queue_for_parsed(), required
- * @import_sync: optional implementation of [method@Importer.import]
  * @import_async: implementation of [method@Importer.import_async], required
  * @import_finish: implementation of [method@Importer.import_finish]
  *
  * Interface implemented for a #GcrImporter.
  */
 
-typedef GcrImporterIface GcrImporterInterface;
-
 G_DEFINE_INTERFACE (GcrImporter, gcr_importer, G_TYPE_OBJECT);
 
 typedef struct _GcrRegistered {
@@ -77,7 +73,7 @@ static GArray *registered_importers = NULL;
 static gboolean registered_sorted = FALSE;
 
 static void
-gcr_importer_default_init (GcrImporterIface *iface)
+gcr_importer_default_init (GcrImporterInterface *iface)
 {
 	static size_t initialized = 0;
 
@@ -93,16 +89,6 @@ gcr_importer_default_init (GcrImporterIface *iface)
 			                     "",
 			                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
-		/**
-		 * GcrImporter:icon:
-		 *
-		 * The icon for the importer.
-		 */
-		g_object_interface_install_property (iface,
-			g_param_spec_object ("icon", "Icon", "The icon for the importer",
-			                     G_TYPE_ICON,
-			                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-
 		/**
 		 * GcrImporter:interaction:
 		 *
@@ -131,11 +117,9 @@ gcr_importer_default_init (GcrImporterIface *iface)
 /**
  * gcr_importer_register:
  * @importer_type: the GType of the importer being registered
- * @attrs: the attributes that this importer is compatible with
+ * @attrs: (transfer full): the attributes that this importer is compatible with
  *
  * Register an importer to handle parsed items that match the given attributes.
- *
- * If @attrs are a floating reference, then it is consumed.
  */
 void
 gcr_importer_register (GType importer_type,
@@ -147,7 +131,7 @@ gcr_importer_register (GType importer_type,
 		registered_importers = g_array_new (FALSE, FALSE, sizeof (GcrRegistered));
 
 	registered.importer_type = importer_type;
-	registered.attrs = gck_attributes_ref_sink (attrs);
+	registered.attrs = attrs;
 	g_array_append_val (registered_importers, registered);
 	registered_sorted = FALSE;
 }
@@ -197,7 +181,7 @@ GList *
 gcr_importer_create_for_parsed (GcrParsed *parsed)
 {
 	GcrRegistered *registered;
-	GcrImporterIface *iface;
+	GcrImporterInterface *iface;
 	gpointer instance_class;
 	GckAttributes *attrs;
 	gboolean matched;
@@ -286,12 +270,12 @@ gboolean
 gcr_importer_queue_for_parsed (GcrImporter *importer,
                                GcrParsed *parsed)
 {
-	GcrImporterIface *iface;
+	GcrImporterInterface *iface;
 
 	g_return_val_if_fail (GCR_IS_IMPORTER (importer), FALSE);
 	g_return_val_if_fail (parsed != NULL, FALSE);
 
-	iface = GCR_IMPORTER_GET_INTERFACE (importer);
+	iface = GCR_IMPORTER_GET_IFACE (importer);
 	g_return_val_if_fail (iface != NULL, FALSE);
 	g_return_val_if_fail (iface->queue_for_parsed != NULL, FALSE);
 
@@ -328,117 +312,6 @@ gcr_importer_queue_and_filter_for_parsed (GList *importers,
 	return g_list_reverse (results);
 }
 
-typedef struct {
-	gboolean complete;
-	GCond *cond;
-	GMutex *mutex;
-	GError *error;
-	GMainContext *context;
-} ImportClosure;
-
-static void
-on_import_async_complete (GObject *source,
-                          GAsyncResult *result,
-                          gpointer user_data)
-{
-	ImportClosure *closure = user_data;
-	GError *error = NULL;
-
-	if (!gcr_importer_import_finish (GCR_IMPORTER (source), result, &error)) {
-		if (error == NULL) {
-			g_warning ("%s::import_finished returned false, but did not set error",
-			           G_OBJECT_TYPE_NAME (source));
-		}
-	}
-
-	g_mutex_lock (closure->mutex);
-
-	closure->complete = TRUE;
-	closure->error = error;
-	g_cond_signal (closure->cond);
-
-	g_mutex_unlock (closure->mutex);
-}
-
-/**
- * gcr_importer_import:
- * @importer: the importer
- * @cancellable: a #GCancellable, or %NULL
- * @error: the location to place an error on failure, or %NULL
- *
- * Import the queued items in the importer. This call will block
- * until the operation completes.
- *
- * Returns: whether the items were imported successfully or not
- */
-gboolean
-gcr_importer_import (GcrImporter *importer,
-                     GCancellable *cancellable,
-                     GError **error)
-{
-	gboolean result;
-	ImportClosure *closure;
-	GcrImporterIface *iface;
-
-	g_return_val_if_fail (GCR_IS_IMPORTER (importer), FALSE);
-	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), FALSE);
-	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-	iface = GCR_IMPORTER_GET_INTERFACE (importer);
-	if (iface->import_sync)
-		return (iface->import_sync) (importer, cancellable, error);
-
-	g_return_val_if_fail (iface->import_async != NULL, FALSE);
-	g_return_val_if_fail (iface->import_finish != NULL, FALSE);
-
-	closure = g_new0 (ImportClosure, 1);
-	closure->cond = g_new (GCond, 1);
-	g_cond_init (closure->cond);
-	closure->mutex = g_new (GMutex, 1);
-	g_mutex_init (closure->mutex);
-	closure->context = g_main_context_get_thread_default ();
-	g_mutex_lock (closure->mutex);
-
-	(iface->import_async) (importer, cancellable, on_import_async_complete, closure);
-
-	/*
-	 * Handle the case where we've been called from within the main context
-	 * or in the case where the main context is not running. This approximates
-	 * the behavior of a modal dialog.
-	 */
-	if (g_main_context_acquire (closure->context)) {
-		while (!closure->complete) {
-			g_mutex_unlock (closure->mutex);
-			g_main_context_iteration (closure->context, TRUE);
-			g_mutex_lock (closure->mutex);
-		}
-
-		g_main_context_release (closure->context);
-
-	/*
-	 * Handle the case where we're in a different thread than the main
-	 * context and a main loop is running.
-	 */
-	} else {
-		while (!closure->complete)
-			g_cond_wait (closure->cond, closure->mutex);
-	}
-
-	g_mutex_unlock (closure->mutex);
-
-	result = (closure->error == NULL);
-	if (closure->error)
-		g_propagate_error (error, closure->error);
-
-	g_cond_clear (closure->cond);
-	g_free (closure->cond);
-	g_mutex_clear (closure->mutex);
-	g_free (closure->mutex);
-	g_free (closure);
-
-	return result;
-}
-
 /**
  * gcr_importer_import_async:
  * @importer: the importer
@@ -455,12 +328,12 @@ gcr_importer_import_async (GcrImporter *importer,
                            GAsyncReadyCallback callback,
                            gpointer user_data)
 {
-	GcrImporterIface *iface;
+	GcrImporterInterface *iface;
 
 	g_return_if_fail (GCR_IS_IMPORTER (importer));
 	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
-	iface = GCR_IMPORTER_GET_INTERFACE (importer);
+	iface = GCR_IMPORTER_GET_IFACE (importer);
 	g_return_if_fail (iface != NULL);
 	g_return_if_fail (iface->import_async != NULL);
 
@@ -482,13 +355,13 @@ gcr_importer_import_finish (GcrImporter *importer,
                             GAsyncResult *result,
                             GError **error)
 {
-	GcrImporterIface *iface;
+	GcrImporterInterface *iface;
 
 	g_return_val_if_fail (GCR_IS_IMPORTER (importer), FALSE);
 	g_return_val_if_fail (G_IS_ASYNC_RESULT (result), FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	iface = GCR_IMPORTER_GET_INTERFACE (importer);
+	iface = GCR_IMPORTER_GET_IFACE (importer);
 	g_return_val_if_fail (iface != NULL, FALSE);
 	g_return_val_if_fail (iface->import_finish != NULL, FALSE);
 
@@ -546,105 +419,3 @@ gcr_importer_register_well_known (void)
 	g_type_class_unref (g_type_class_ref (GCR_TYPE_PKCS11_IMPORTER));
 	g_type_class_unref (g_type_class_ref (GCR_TYPE_GNUPG_IMPORTER));
 }
-
-#ifndef GCR_DISABLE_DEPRECATED
-
-/**
- * gcr_importer_get_parser:
- * @self: An importer
- *
- * Has no effect. Use gcr_importer_listen() instead.
- *
- * Returns: %NULL is always returned.
- * Deprecated: Since 3.0.0
- */
-GcrParser*
-gcr_importer_get_parser (GcrImporter *self)
-{
-        g_warning ("gcr_importer_get_parser() is no longer supported "
-                   "Use gcr_importer_listen() instead.");
-        return NULL;
-}
-
-/**
- * gcr_importer_set_parser:
- * @self: An importer
- * @parser: A parser
- *
- * Has no effect. Use gcr_importer_listen() instead.
- *
- * Deprecated: Since 3.0.0
- */
-void
-gcr_importer_set_parser (GcrImporter *self,
-                         GcrParser *parser)
-{
-        g_warning ("gcr_importer_set_parser() is no longer supported "
-                   "Use gcr_importer_listen() instead.");
-}
-
-/*
- * gcr_importer_get_slot:
- * @self: The importer
- *
- * Returns %NULL.
- *
- * Deprecated: since 3.4.0
- */
-GckSlot *
-gcr_importer_get_slot (GcrImporter *self)
-{
-        g_warning ("gcr_importer_get_slot() is no longer supported.");
-        return NULL;
-}
-
-/**
- * gcr_importer_set_slot:
- * @self: The importer
- * @slot: The slot to import to
- *
- * Has no effect.
- *
- * Deprecated: since 3.4.0
- */
-void
-gcr_importer_set_slot (GcrImporter *self,
-                       GckSlot *slot)
-{
-        g_warning ("gcr_importer_set_slot() is no longer supported.");
-}
-
-/**
- * gcr_importer_get_prompt_behavior:
- * @self: The importer
- *
- * Does nothing.
- *
- * Returns: zero
- *
- * Deprecated: since 3.4.0
- */
-GcrImporterPromptBehavior
-gcr_importer_get_prompt_behavior (GcrImporter *self)
-{
-	g_warning ("gcr_importer_get_prompt_behavior() is no longer supported.");
-	return 0;
-}
-
-/**
- * gcr_importer_set_prompt_behavior:
- * @self: The importer
- * @behavior: The prompt behavior flag
- *
- * Has no effect.
- *
- * Deprecated: since 3.4.0
- */
-void
-gcr_importer_set_prompt_behavior (GcrImporter *self,
-                                  GcrImporterPromptBehavior behavior)
-{
-	g_warning ("gcr_importer_set_prompt_behavior() is no longer supported.");
-}
-
-#endif /* GCR_DISABLE_DEPRECATED */
