@@ -23,7 +23,6 @@
 #include "config.h"
 
 #include "gcr-fingerprint.h"
-#include "gcr-icons.h"
 #include "gcr-internal.h"
 #include "gcr-library.h"
 #include "gcr-import-interaction.h"
@@ -42,7 +41,6 @@
 enum {
 	PROP_0,
 	PROP_LABEL,
-	PROP_ICON,
 	PROP_INTERACTION,
 	PROP_SLOT,
 	PROP_IMPORTED,
@@ -72,7 +70,7 @@ static void   state_cancelled                  (GTask *task,
                                                 gboolean async);
 static void   state_create_object              (GTask *task,
                                                 gboolean async);
-static void   _gcr_pkcs11_importer_init_iface  (GcrImporterIface *iface);
+static void   _gcr_pkcs11_importer_init_iface  (GcrImporterInterface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (GcrPkcs11Importer, _gcr_pkcs11_importer, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (GCR_TYPE_IMPORTER, _gcr_pkcs11_importer_init_iface);
@@ -306,7 +304,7 @@ supplement_attributes (GcrPkcs11Importer *self,
 		}
 
 		gck_attributes_unref (attrs);
-		l->data = attrs = gck_attributes_ref_sink (gck_builder_end (&builder));
+		l->data = attrs = gck_builder_end (&builder);
 
 		switch (klass) {
 		case CKO_CERTIFICATE:
@@ -340,14 +338,14 @@ supplement_attributes (GcrPkcs11Importer *self,
 			supplement_with_attributes (&builder, supplements);
 			supplement_id_for_data (&builder, nonce, sizeof (nonce),
 			                        fingerprint, strlen (fingerprint));
-			g_queue_push_tail (queue, gck_attributes_ref_sink (gck_builder_end (&builder)));
+			g_queue_push_tail (queue, gck_builder_end (&builder));
 			g_hash_table_insert (paired, pair->private_key, "present");
 
 			gck_builder_add_all (&builder, pair->certificate);
 			supplement_with_attributes (&builder, supplements);
 			supplement_id_for_data (&builder, nonce, sizeof (nonce),
 			                        fingerprint, strlen (fingerprint));
-			g_queue_push_tail (queue, gck_attributes_ref_sink (gck_builder_end (&builder)));
+			g_queue_push_tail (queue, gck_builder_end (&builder));
 			g_hash_table_insert (paired, pair->certificate, "present");
 
 			/* Used the suplements for the pairs, don't use for unpaired stuff */
@@ -371,7 +369,7 @@ supplement_attributes (GcrPkcs11Importer *self,
 			supplement_id_for_data (&builder, nonce, sizeof (nonce),
 			                        &attrs, sizeof (gpointer));
 
-			g_queue_push_tail (queue, gck_attributes_ref_sink (gck_builder_end (&builder)));
+			g_queue_push_tail (queue, gck_builder_end (&builder));
 		}
 	}
 
@@ -392,7 +390,7 @@ complete_supplement (GTask *task,
 	GckAttributes *attributes;
 
 	if (error == NULL) {
-		attributes = gck_attributes_ref_sink (gck_builder_end (data->supplement));
+		attributes = gck_builder_end (data->supplement);
 		supplement_attributes (data->importer, attributes);
 		gck_attributes_unref (attributes);
 
@@ -605,21 +603,6 @@ calculate_label (GcrPkcs11Importer *self)
 	return result;
 }
 
-static GIcon *
-calculate_icon (GcrPkcs11Importer *self,
-                GckTokenInfo *token_info)
-{
-	GckTokenInfo *info = NULL;
-	GIcon *result;
-
-	if (token_info == NULL)
-		info = token_info = gck_slot_get_token_info (self->slot);
-	result = gcr_icon_for_token (token_info);
-	gck_token_info_free (info);
-
-	return result;
-}
-
 static gchar *
 calculate_uri (GcrPkcs11Importer *self)
 {
@@ -628,7 +611,7 @@ calculate_uri (GcrPkcs11Importer *self)
 
 	data = gck_uri_data_new ();
 	data->token_info = gck_slot_get_token_info (self->slot);
-	uri = gck_uri_build (data, GCK_URI_FOR_TOKEN);
+	uri = gck_uri_data_build (data, GCK_URI_FOR_TOKEN);
 	data->token_info = NULL;
 	gck_uri_data_free (data);
 
@@ -647,9 +630,6 @@ _gcr_pkcs11_importer_get_property (GObject *obj,
 	case PROP_LABEL:
 		g_value_take_string (value, calculate_label (self));
 		break;
-	case PROP_ICON:
-		g_value_take_object (value, calculate_icon (self, NULL));
-		break;
 	case PROP_SLOT:
 		g_value_set_object (value, _gcr_pkcs11_importer_get_slot (self));
 		break;
@@ -683,7 +663,6 @@ _gcr_pkcs11_importer_class_init (GcrPkcs11ImporterClass *klass)
 	gobject_class->get_property = _gcr_pkcs11_importer_get_property;
 
 	g_object_class_override_property (gobject_class, PROP_LABEL, "label");
-	g_object_class_override_property (gobject_class, PROP_ICON, "icon");
 	g_object_class_override_property (gobject_class, PROP_INTERACTION, "interaction");
 	g_object_class_override_property (gobject_class, PROP_URI, "uri");
 
@@ -754,7 +733,7 @@ is_slot_importable (GckSlot *slot,
 	}
 
 	for (i = 0; token_blacklist[i] != NULL; i++) {
-		uri = gck_uri_parse (token_blacklist[i], GCK_URI_FOR_TOKEN | GCK_URI_FOR_MODULE, &error);
+		uri = gck_uri_data_parse (token_blacklist[i], GCK_URI_FOR_TOKEN | GCK_URI_FOR_MODULE, &error);
 		if (uri == NULL) {
 			g_warning ("couldn't parse pkcs11 blacklist uri: %s", error->message);
 			g_clear_error (&error);
@@ -851,7 +830,7 @@ _gcr_pkcs11_importer_import_finish (GcrImporter *importer,
 }
 
 static void
-_gcr_pkcs11_importer_init_iface (GcrImporterIface *iface)
+_gcr_pkcs11_importer_init_iface (GcrImporterInterface *iface)
 {
 	iface->create_for_parsed = _gcr_pkcs11_importer_create_for_parsed;
 	iface->queue_for_parsed = _gcr_pkcs11_importer_queue_for_parsed;
@@ -911,7 +890,9 @@ _gcr_pkcs11_importer_queue (GcrPkcs11Importer *self,
 		gck_builder_add_all (&builder, attrs);
 		gck_builder_add_string (&builder, CKA_LABEL, label);
 		attrs = gck_builder_end (&builder);
+	} else {
+		gck_attributes_ref (attrs);
 	}
 
-	g_queue_push_tail (self->queue, gck_attributes_ref_sink (attrs));
+	g_queue_push_tail (self->queue, attrs);
 }
